<div class="row">
    <div class="col-md-12">
        <div class="col-xs-6" id="data-title">
            <h2>{{ data.data.title }} <small> {{ data.data.template.display_name }}</small></h2>
        </div>
        {% include 'core_main_app/common/data/tools_data.html' with template_xsl_rendering=data.template_xsl_rendering xslt_id=data.xsl_transformation_id can_display_selector=data.can_display_selector%}
    </div>
</div>
{% include 'core_main_app/common/data/detail_data.html'%}